@@ -5,50 +5,87 @@
 from django.contrib.auth.models import User
 from django.contrib.sites.models import Site
 from djblets.siteconfig.models import SiteConfiguration
+from djblets.util.decorators import simple_decorator
 from reviewboard.reviews.errors import PublishError
 from reviewboard.reviews.signals import (review_request_publishing,
                                          review_publishing, reply_publishing)
 from reviewboard.site.urlresolvers import local_site_reverse
 
 from rbbz.bugzilla import Bugzilla
-from rbbz.errors import BugzillaError, InvalidBugIdError, InvalidReviewerError
+from rbbz.errors import (BugzillaError, InvalidBugsError, InvalidBugIdError,
+                         InvalidReviewersError)
 
-def publish_review_request(user, review_request_draft, **kwargs):
-    try:
-        bug_id = int(review_request_draft.get_bug_list()[0])
-    except (IndexError, TypeError, ValueError):
-        raise InvalidBugIdError
+def review_request_url(review_request, site=None, siteconfig=None):
+    if not site:
+        site = Site.objects.get_current()
 
-    reviewer = review_request_draft.target_people.first()
-    if reviewer is None:
-        raise InvalidReviewerError
+    if not siteconfig:
+        siteconfig = SiteConfiguration.objects.get_current()
 
-    site = Site.objects.get_current()
-    siteconfig = SiteConfiguration.objects.get_current()
-
-    url = '%s://%s%s%s' % (
+    return '%s://%s%s%s' % (
         siteconfig.get('site_domain_method'), site.domain,
         local_site_reverse('root').rstrip('/'),
-        review_request_draft.get_review_request().get_absolute_url())
+        review_request.get_absolute_url())
+
+
+@simple_decorator
+def bugzilla_to_publish_errors(func):
+    def _transform_errors(*args, **kwargs):
+        try:
+            return func(*args, **kwargs)
+        except BugzillaError as e:
+            raise PublishError('Bugzilla error: %s' % e.msg)
+    return _transform_errors
+
+
+@bugzilla_to_publish_errors
+def publish_review_request(user, review_request_draft, **kwargs):
+    bugs = review_request_draft.get_bug_list()
+
+    if len(bugs) == 0 or len(bugs) > 1:
+        raise InvalidBugsError
 
     try:
-        b = Bugzilla(user.bzlogin, user.bzcookie)
-        b.post_rb_url(review_request_draft.summary, bug_id, url,
-                      reviewer.get_username())
-    except BugzillaError as e:
-        raise PublishError(e.msg)
+        bug_id = int(bugs[0])
+    except (TypeError, ValueError):
+        raise InvalidBugIdError(bugs[0])
 
+    reviewers = review_request_draft.target_people
+    num_reviewers = reviewers.count()
+
+    if num_reviewers == 0 or num_reviewers > 1:
+        raise InvalidReviewersError
+
+    reviewer = reviewers.first()
+    b = Bugzilla(user.bzlogin, user.bzcookie)
+    b.post_rb_url(review_request_draft.summary, bug_id,
+                  review_request_url(review_request_draft.get_review_request()),
+                  reviewer.get_username())
 
-review_request_publishing.connect(publish_review_request)
 
 def publish_review(user, review, **kwargs):
-    pass
+    if review.ship_it:
+        bug_id = int(review.review_request.get_bug_list()[0])
+        site = Site.objects.get_current()
+        siteconfig = SiteConfiguration.objects.get_current()
+
+        b = Bugzilla(user.bzlogin, user.bzcookie)
+        attachments = b.get_rb_attachments(bug_id)
+
+        for a in attachments:
+            if (a['reviewer'] == review.user.username and
+                a['url'] == review_request_url(review.review_request, site,
+                                               siteconfig)):
+                b.r_plus_attachment(a['id'])
+                break
 
-review_publishing.connect(publish_review)
 
 def publish_reply(user, reply, **kwargs):
     pass
 
+
+review_request_publishing.connect(publish_review_request)
+review_publishing.connect(publish_review)
 reply_publishing.connect(publish_reply)
 
 
